# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
from google.type import date_pb2  # type: ignore
from google.type import decimal_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "RebillingBasis",
        "CustomerRepricingConfig",
        "ChannelPartnerRepricingConfig",
        "RepricingConfig",
        "RepricingAdjustment",
        "PercentageAdjustment",
        "ConditionalOverride",
        "RepricingCondition",
        "SkuGroupCondition",
    },
)


class RebillingBasis(proto.Enum):
    r"""Specifies the different costs that the modified bill can be
    based on.

    Values:
        REBILLING_BASIS_UNSPECIFIED (0):
            Not used.
        COST_AT_LIST (1):
            Use the list cost, also known as the MSRP.
        DIRECT_CUSTOMER_COST (2):
            Pass through all discounts except the
            Reseller Program Discount. If this is the
            default cost base and no adjustments are
            specified, the output cost will be exactly what
            the customer would see if they viewed the bill
            in the Google Cloud Console.
    """
    REBILLING_BASIS_UNSPECIFIED = 0
    COST_AT_LIST = 1
    DIRECT_CUSTOMER_COST = 2


class CustomerRepricingConfig(proto.Message):
    r"""Configuration for how a reseller will reprice a Customer.

    Attributes:
        name (str):
            Output only. Resource name of the CustomerRepricingConfig.
            Format:
            accounts/{account_id}/customers/{customer_id}/customerRepricingConfigs/{id}.
        repricing_config (google.cloud.channel_v1.types.RepricingConfig):
            Required. The configuration for bill
            modifications made by a reseller before sending
            it to customers.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp of an update to the repricing rule.
            If ``update_time`` is after
            [RepricingConfig.effective_invoice_month][google.cloud.channel.v1.RepricingConfig.effective_invoice_month]
            then it indicates this was set mid-month.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    repricing_config: "RepricingConfig" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="RepricingConfig",
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class ChannelPartnerRepricingConfig(proto.Message):
    r"""Configuration for how a distributor will rebill a channel
    partner (also known as a distributor-authorized reseller).

    Attributes:
        name (str):
            Output only. Resource name of the
            ChannelPartnerRepricingConfig. Format:
            accounts/{account_id}/channelPartnerLinks/{channel_partner_id}/channelPartnerRepricingConfigs/{id}.
        repricing_config (google.cloud.channel_v1.types.RepricingConfig):
            Required. The configuration for bill
            modifications made by a reseller before sending
            it to ChannelPartner.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp of an update to the repricing rule.
            If ``update_time`` is after
            [RepricingConfig.effective_invoice_month][google.cloud.channel.v1.RepricingConfig.effective_invoice_month]
            then it indicates this was set mid-month.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    repricing_config: "RepricingConfig" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="RepricingConfig",
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class RepricingConfig(proto.Message):
    r"""Configuration for repricing a Google bill over a period of
    time.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        entitlement_granularity (google.cloud.channel_v1.types.RepricingConfig.EntitlementGranularity):
            Applies the repricing configuration at the entitlement
            level.

            Note: If a
            [ChannelPartnerRepricingConfig][google.cloud.channel.v1.ChannelPartnerRepricingConfig]
            using
            [RepricingConfig.EntitlementGranularity][google.cloud.channel.v1.RepricingConfig.EntitlementGranularity]
            becomes effective, then no existing or future
            [RepricingConfig.ChannelPartnerGranularity][google.cloud.channel.v1.RepricingConfig.ChannelPartnerGranularity]
            will apply to the
            [RepricingConfig.EntitlementGranularity.entitlement][google.cloud.channel.v1.RepricingConfig.EntitlementGranularity.entitlement].
            This is the recommended value for both
            [CustomerRepricingConfig][google.cloud.channel.v1.CustomerRepricingConfig]
            and
            [ChannelPartnerRepricingConfig][google.cloud.channel.v1.ChannelPartnerRepricingConfig].

            This field is a member of `oneof`_ ``granularity``.
        channel_partner_granularity (google.cloud.channel_v1.types.RepricingConfig.ChannelPartnerGranularity):
            Applies the repricing configuration at the channel partner
            level. Only
            [ChannelPartnerRepricingConfig][google.cloud.channel.v1.ChannelPartnerRepricingConfig]
            supports this value. Deprecated: This is no longer
            supported. Use
            [RepricingConfig.entitlement_granularity][google.cloud.channel.v1.RepricingConfig.entitlement_granularity]
            instead.

            This field is a member of `oneof`_ ``granularity``.
        effective_invoice_month (google.type.date_pb2.Date):
            Required. The YearMonth when these
            adjustments activate. The Day field needs to be
            "0" since we only accept YearMonth repricing
            boundaries.
        adjustment (google.cloud.channel_v1.types.RepricingAdjustment):
            Required. Information about the adjustment.
        rebilling_basis (google.cloud.channel_v1.types.RebillingBasis):
            Required. The
            [RebillingBasis][google.cloud.channel.v1.RebillingBasis] to
            use for this bill. Specifies the relative cost based on
            repricing costs you will apply.
        conditional_overrides (MutableSequence[google.cloud.channel_v1.types.ConditionalOverride]):
            The conditional overrides to apply for this
            configuration. If you list multiple overrides,
            only the first valid override is used.  If you
            don't list any overrides, the API uses the
            normal adjustment and rebilling basis.
    """

    class EntitlementGranularity(proto.Message):
        r"""Applies the repricing configuration at the entitlement level.

        Attributes:
            entitlement (str):
                Resource name of the entitlement. Format:
                accounts/{account_id}/customers/{customer_id}/entitlements/{entitlement_id}
        """

        entitlement: str = proto.Field(
            proto.STRING,
            number=1,
        )

    class ChannelPartnerGranularity(proto.Message):
        r"""Applies the repricing configuration at the channel partner level.
        The channel partner value is derived from the resource name. Takes
        an empty json object. Deprecated: This is no longer supported. Use
        [RepricingConfig.EntitlementGranularity][google.cloud.channel.v1.RepricingConfig.EntitlementGranularity]
        instead.

        """

    entitlement_granularity: EntitlementGranularity = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="granularity",
        message=EntitlementGranularity,
    )
    channel_partner_granularity: ChannelPartnerGranularity = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="granularity",
        message=ChannelPartnerGranularity,
    )
    effective_invoice_month: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=1,
        message=date_pb2.Date,
    )
    adjustment: "RepricingAdjustment" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="RepricingAdjustment",
    )
    rebilling_basis: "RebillingBasis" = proto.Field(
        proto.ENUM,
        number=3,
        enum="RebillingBasis",
    )
    conditional_overrides: MutableSequence["ConditionalOverride"] = proto.RepeatedField(
        proto.MESSAGE,
        number=6,
        message="ConditionalOverride",
    )


class RepricingAdjustment(proto.Message):
    r"""A type that represents the various adjustments you can apply
    to a bill.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        percentage_adjustment (google.cloud.channel_v1.types.PercentageAdjustment):
            Flat markup or markdown on an entire bill.

            This field is a member of `oneof`_ ``adjustment``.
    """

    percentage_adjustment: "PercentageAdjustment" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="adjustment",
        message="PercentageAdjustment",
    )


class PercentageAdjustment(proto.Message):
    r"""An adjustment that applies a flat markup or markdown to an
    entire bill.

    Attributes:
        percentage (google.type.decimal_pb2.Decimal):
            The percentage of the bill to adjust.
            For example:

            Mark down by 1% => "-1.00"
            Mark up by 1%   => "1.00"
            Pass-Through    => "0.00".
    """

    percentage: decimal_pb2.Decimal = proto.Field(
        proto.MESSAGE,
        number=2,
        message=decimal_pb2.Decimal,
    )


class ConditionalOverride(proto.Message):
    r"""Specifies the override to conditionally apply.

    Attributes:
        adjustment (google.cloud.channel_v1.types.RepricingAdjustment):
            Required. Information about the applied
            override's adjustment.
        rebilling_basis (google.cloud.channel_v1.types.RebillingBasis):
            Required. The
            [RebillingBasis][google.cloud.channel.v1.RebillingBasis] to
            use for the applied override. Shows the relative cost based
            on your repricing costs.
        repricing_condition (google.cloud.channel_v1.types.RepricingCondition):
            Required. Specifies the condition which, if
            met, will apply the override.
    """

    adjustment: "RepricingAdjustment" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="RepricingAdjustment",
    )
    rebilling_basis: "RebillingBasis" = proto.Field(
        proto.ENUM,
        number=2,
        enum="RebillingBasis",
    )
    repricing_condition: "RepricingCondition" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="RepricingCondition",
    )


class RepricingCondition(proto.Message):
    r"""Represents the various repricing conditions you can use for a
    conditional override.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        sku_group_condition (google.cloud.channel_v1.types.SkuGroupCondition):
            SKU Group condition for override.

            This field is a member of `oneof`_ ``condition``.
    """

    sku_group_condition: "SkuGroupCondition" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="condition",
        message="SkuGroupCondition",
    )


class SkuGroupCondition(proto.Message):
    r"""A condition that applies the override if a line item SKU is
    found in the SKU group.

    Attributes:
        sku_group (str):
            Specifies a SKU group
            (https://cloud.google.com/skus/sku-groups). Resource name of
            SKU group. Format: accounts/{account}/skuGroups/{sku_group}.
            Example:
            "accounts/C01234/skuGroups/3d50fd57-3157-4577-a5a9-a219b8490041".
    """

    sku_group: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
from google.type import date_pb2  # type: ignore
from google.type import datetime_pb2  # type: ignore
from google.type import decimal_pb2  # type: ignore
from google.type import money_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "RunReportJobRequest",
        "RunReportJobResponse",
        "FetchReportResultsRequest",
        "FetchReportResultsResponse",
        "ListReportsRequest",
        "ListReportsResponse",
        "ReportJob",
        "ReportResultsMetadata",
        "Column",
        "DateRange",
        "Row",
        "ReportValue",
        "ReportStatus",
        "Report",
    },
)


class RunReportJobRequest(proto.Message):
    r"""Request message for
    [CloudChannelReportsService.RunReportJob][google.cloud.channel.v1.CloudChannelReportsService.RunReportJob].

    Attributes:
        name (str):
            Required. The report's resource name. Specifies the account
            and report used to generate report data. The report_id
            identifier is a UID (for example, ``613bf59q``). Name uses
            the format: accounts/{account_id}/reports/{report_id}
        date_range (google.cloud.channel_v1.types.DateRange):
            Optional. The range of usage or invoice dates
            to include in the result.
        filter (str):
            Optional. A structured string that defines conditions on
            dimension columns to restrict the report output.

            Filters support logical operators (AND, OR, NOT) and
            conditional operators (=, !=, <, >, <=, and >=) using
            ``column_id`` as keys.

            For example:
            ``(customer:"accounts/C123abc/customers/S456def" OR customer:"accounts/C123abc/customers/S789ghi") AND invoice_start_date.year >= 2022``
        language_code (str):
            Optional. The BCP-47 language code, such as
            "en-US".  If specified, the response is
            localized to the corresponding language code if
            the original data sources support it.
            Default is "en-US".
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    date_range: "DateRange" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="DateRange",
    )
    filter: str = proto.Field(
        proto.STRING,
        number=3,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=4,
    )


class RunReportJobResponse(proto.Message):
    r"""Response message for
    [CloudChannelReportsService.RunReportJob][google.cloud.channel.v1.CloudChannelReportsService.RunReportJob].

    Attributes:
        report_job (google.cloud.channel_v1.types.ReportJob):
            Pass ``report_job.name`` to
            [FetchReportResultsRequest.report_job][google.cloud.channel.v1.FetchReportResultsRequest.report_job]
            to retrieve the report's results.
        report_metadata (google.cloud.channel_v1.types.ReportResultsMetadata):
            The metadata for the report's results
            (display name, columns, row count, and date
            range). If you view this before the operation
            finishes, you may see incomplete data.
    """

    report_job: "ReportJob" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="ReportJob",
    )
    report_metadata: "ReportResultsMetadata" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ReportResultsMetadata",
    )


class FetchReportResultsRequest(proto.Message):
    r"""Request message for
    [CloudChannelReportsService.FetchReportResults][google.cloud.channel.v1.CloudChannelReportsService.FetchReportResults].

    Attributes:
        report_job (str):
            Required. The report job created by
            [CloudChannelReportsService.RunReportJob][google.cloud.channel.v1.CloudChannelReportsService.RunReportJob].
            Report_job uses the format:
            accounts/{account_id}/reportJobs/{report_job_id}
        page_size (int):
            Optional. Requested page size of the report.
            The server may return fewer results than
            requested. If you don't specify a page size, the
            server uses a sensible default (may change over
            time).

            The maximum value is 30,000; the server will
            change larger values to 30,000.
        page_token (str):
            Optional. A token that specifies a page of results beyond
            the first page. Obtained through
            [FetchReportResultsResponse.next_page_token][google.cloud.channel.v1.FetchReportResultsResponse.next_page_token]
            of the previous
            [CloudChannelReportsService.FetchReportResults][google.cloud.channel.v1.CloudChannelReportsService.FetchReportResults]
            call.
        partition_keys (MutableSequence[str]):
            Optional. List of keys specifying which
            report partitions to return. If empty, returns
            all partitions.
    """

    report_job: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    partition_keys: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )


class FetchReportResultsResponse(proto.Message):
    r"""Response message for
    [CloudChannelReportsService.FetchReportResults][google.cloud.channel.v1.CloudChannelReportsService.FetchReportResults].
    Contains a tabular representation of the report results.

    Attributes:
        report_metadata (google.cloud.channel_v1.types.ReportResultsMetadata):
            The metadata for the report results (display
            name, columns, row count, and date ranges).
        rows (MutableSequence[google.cloud.channel_v1.types.Row]):
            The report's lists of values. Each row follows the settings
            and ordering of the columns from ``report_metadata``.
        next_page_token (str):
            Pass this token to
            [FetchReportResultsRequest.page_token][google.cloud.channel.v1.FetchReportResultsRequest.page_token]
            to retrieve the next page of results.
    """

    @property
    def raw_page(self):
        return self

    report_metadata: "ReportResultsMetadata" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="ReportResultsMetadata",
    )
    rows: MutableSequence["Row"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="Row",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListReportsRequest(proto.Message):
    r"""Request message for
    [CloudChannelReportsService.ListReports][google.cloud.channel.v1.CloudChannelReportsService.ListReports].

    Attributes:
        parent (str):
            Required. The resource name of the partner account to list
            available reports for. Parent uses the format:
            accounts/{account_id}
        page_size (int):
            Optional. Requested page size of the report.
            The server might return fewer results than
            requested. If unspecified, returns 20 reports.
            The maximum value is 100.
        page_token (str):
            Optional. A token that specifies a page of results beyond
            the first page. Obtained through
            [ListReportsResponse.next_page_token][google.cloud.channel.v1.ListReportsResponse.next_page_token]
            of the previous
            [CloudChannelReportsService.ListReports][google.cloud.channel.v1.CloudChannelReportsService.ListReports]
            call.
        language_code (str):
            Optional. The BCP-47 language code, such as
            "en-US".  If specified, the response is
            localized to the corresponding language code if
            the original data sources support it.
            Default is "en-US".
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListReportsResponse(proto.Message):
    r"""Response message for
    [CloudChannelReportsService.ListReports][google.cloud.channel.v1.CloudChannelReportsService.ListReports].

    Attributes:
        reports (MutableSequence[google.cloud.channel_v1.types.Report]):
            The reports available to the partner.
        next_page_token (str):
            Pass this token to
            [FetchReportResultsRequest.page_token][google.cloud.channel.v1.FetchReportResultsRequest.page_token]
            to retrieve the next page of results.
    """

    @property
    def raw_page(self):
        return self

    reports: MutableSequence["Report"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Report",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ReportJob(proto.Message):
    r"""The result of a [RunReportJob][] operation. Contains the name to use
    in
    [FetchReportResultsRequest.report_job][google.cloud.channel.v1.FetchReportResultsRequest.report_job]
    and the status of the operation.

    Attributes:
        name (str):
            Required. The resource name of a report job. Name uses the
            format: ``accounts/{account_id}/reportJobs/{report_job_id}``
        report_status (google.cloud.channel_v1.types.ReportStatus):
            The current status of report generation.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    report_status: "ReportStatus" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ReportStatus",
    )


class ReportResultsMetadata(proto.Message):
    r"""The features describing the data. Returned by
    [CloudChannelReportsService.RunReportJob][google.cloud.channel.v1.CloudChannelReportsService.RunReportJob]
    and
    [CloudChannelReportsService.FetchReportResults][google.cloud.channel.v1.CloudChannelReportsService.FetchReportResults].

    Attributes:
        report (google.cloud.channel_v1.types.Report):
            Details of the completed report.
        row_count (int):
            The total number of rows of data in the final
            report.
        date_range (google.cloud.channel_v1.types.DateRange):
            The date range of reported usage.
        preceding_date_range (google.cloud.channel_v1.types.DateRange):
            The usage dates immediately preceding ``date_range`` with
            the same duration. Use this to calculate trending usage and
            costs. This is only populated if you request trending data.

            For example, if ``date_range`` is July 1-15,
            ``preceding_date_range`` will be June 16-30.
    """

    report: "Report" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="Report",
    )
    row_count: int = proto.Field(
        proto.INT64,
        number=2,
    )
    date_range: "DateRange" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="DateRange",
    )
    preceding_date_range: "DateRange" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="DateRange",
    )


class Column(proto.Message):
    r"""The definition of a report column. Specifies the data
    properties in the corresponding position of the report rows.

    Attributes:
        column_id (str):
            The unique name of the column (for example, customer_domain,
            channel_partner, customer_cost). You can use column IDs in
            [RunReportJobRequest.filter][google.cloud.channel.v1.RunReportJobRequest.filter].
            To see all reports and their columns, call
            [CloudChannelReportsService.ListReports][google.cloud.channel.v1.CloudChannelReportsService.ListReports].
        display_name (str):
            The column's display name.
        data_type (google.cloud.channel_v1.types.Column.DataType):
            The type of the values for this column.
    """

    class DataType(proto.Enum):
        r"""Available data types for columns. Corresponds to the fields in the
        ReportValue ``oneof`` field.

        Values:
            DATA_TYPE_UNSPECIFIED (0):
                Not used.
            STRING (1):
                ReportValues for this column will use string_value.
            INT (2):
                ReportValues for this column will use int_value.
            DECIMAL (3):
                ReportValues for this column will use decimal_value.
            MONEY (4):
                ReportValues for this column will use money_value.
            DATE (5):
                ReportValues for this column will use date_value.
            DATE_TIME (6):
                ReportValues for this column will use date_time_value.
        """
        DATA_TYPE_UNSPECIFIED = 0
        STRING = 1
        INT = 2
        DECIMAL = 3
        MONEY = 4
        DATE = 5
        DATE_TIME = 6

    column_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    data_type: DataType = proto.Field(
        proto.ENUM,
        number=3,
        enum=DataType,
    )


class DateRange(proto.Message):
    r"""A representation of usage or invoice date ranges.

    Attributes:
        usage_start_date_time (google.type.datetime_pb2.DateTime):
            The earliest usage date time (inclusive).

            If you use time groupings (daily, weekly, etc), each group
            uses midnight to midnight (Pacific time). The usage start
            date is rounded down to include all usage from the specified
            date. We recommend that clients pass
            ``usage_start_date_time`` in Pacific time.
        usage_end_date_time (google.type.datetime_pb2.DateTime):
            The latest usage date time (exclusive).

            If you use time groupings (daily, weekly, etc), each group
            uses midnight to midnight (Pacific time). The usage end date
            is rounded down to include all usage from the specified
            date. We recommend that clients pass
            ``usage_start_date_time`` in Pacific time.
        invoice_start_date (google.type.date_pb2.Date):
            The earliest invoice date (inclusive).

            If this value is not the first day of a month,
            this will move it back to the first day of the
            given month.
        invoice_end_date (google.type.date_pb2.Date):
            The latest invoice date (inclusive).

            If this value is not the last day of a month,
            this will move it forward to the last day of the
            given month.
    """

    usage_start_date_time: datetime_pb2.DateTime = proto.Field(
        proto.MESSAGE,
        number=1,
        message=datetime_pb2.DateTime,
    )
    usage_end_date_time: datetime_pb2.DateTime = proto.Field(
        proto.MESSAGE,
        number=2,
        message=datetime_pb2.DateTime,
    )
    invoice_start_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=3,
        message=date_pb2.Date,
    )
    invoice_end_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=4,
        message=date_pb2.Date,
    )


class Row(proto.Message):
    r"""A row of report values.

    Attributes:
        values (MutableSequence[google.cloud.channel_v1.types.ReportValue]):
            The list of values in the row.
        partition_key (str):
            The key for the partition this row belongs
            to. This field is empty if the report is not
            partitioned.
    """

    values: MutableSequence["ReportValue"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ReportValue",
    )
    partition_key: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ReportValue(proto.Message):
    r"""A single report value.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        string_value (str):
            A value of type ``string``.

            This field is a member of `oneof`_ ``value``.
        int_value (int):
            A value of type ``int``.

            This field is a member of `oneof`_ ``value``.
        decimal_value (google.type.decimal_pb2.Decimal):
            A value of type ``google.type.Decimal``, representing
            non-integer numeric values.

            This field is a member of `oneof`_ ``value``.
        money_value (google.type.money_pb2.Money):
            A value of type ``google.type.Money`` (currency code, whole
            units, decimal units).

            This field is a member of `oneof`_ ``value``.
        date_value (google.type.date_pb2.Date):
            A value of type ``google.type.Date`` (year, month, day).

            This field is a member of `oneof`_ ``value``.
        date_time_value (google.type.datetime_pb2.DateTime):
            A value of type ``google.type.DateTime`` (year, month, day,
            hour, minute, second, and UTC offset or timezone.)

            This field is a member of `oneof`_ ``value``.
    """

    string_value: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="value",
    )
    int_value: int = proto.Field(
        proto.INT64,
        number=2,
        oneof="value",
    )
    decimal_value: decimal_pb2.Decimal = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="value",
        message=decimal_pb2.Decimal,
    )
    money_value: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="value",
        message=money_pb2.Money,
    )
    date_value: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="value",
        message=date_pb2.Date,
    )
    date_time_value: datetime_pb2.DateTime = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="value",
        message=datetime_pb2.DateTime,
    )


class ReportStatus(proto.Message):
    r"""Status of a report generation process.

    Attributes:
        state (google.cloud.channel_v1.types.ReportStatus.State):
            The current state of the report generation
            process.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The report generation's start time.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            The report generation's completion time.
    """

    class State(proto.Enum):
        r"""Available states of report generation.

        Values:
            STATE_UNSPECIFIED (0):
                Not used.
            STARTED (1):
                Report processing started.
            WRITING (2):
                Data generated from the report is being
                staged.
            AVAILABLE (3):
                Report data is available for access.
            FAILED (4):
                Report failed.
        """
        STATE_UNSPECIFIED = 0
        STARTED = 1
        WRITING = 2
        AVAILABLE = 3
        FAILED = 4

    state: State = proto.Field(
        proto.ENUM,
        number=1,
        enum=State,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class Report(proto.Message):
    r"""The ID and description of a report that was used to generate
    report data. For example, "Google Cloud Daily Spend", "Google
    Workspace License Activity", etc.

    Attributes:
        name (str):
            Required. The report's resource name. Specifies the account
            and report used to generate report data. The report_id
            identifier is a UID (for example, ``613bf59q``).

            Name uses the format:
            accounts/{account_id}/reports/{report_id}
        display_name (str):
            A human-readable name for this report.
        columns (MutableSequence[google.cloud.channel_v1.types.Column]):
            The list of columns included in the report.
            This defines the schema of the report results.
        description (str):
            A description of other aspects of the report,
            such as the products it supports.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    columns: MutableSequence["Column"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="Column",
    )
    description: str = proto.Field(
        proto.STRING,
        number=4,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import (
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.channel_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object]  # type: ignore

from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.longrunning import operations_pb2  # type: ignore

from google.cloud.channel_v1.services.cloud_channel_reports_service import pagers
from google.cloud.channel_v1.types import operations, reports_service

from .client import CloudChannelReportsServiceClient
from .transports.base import DEFAULT_CLIENT_INFO, CloudChannelReportsServiceTransport
from .transports.grpc_asyncio import CloudChannelReportsServiceGrpcAsyncIOTransport


class CloudChannelReportsServiceAsyncClient:
    """CloudChannelReportsService lets Google Cloud resellers and
    distributors retrieve and combine a variety of data in Cloud
    Channel for multiple products (Google Cloud, Google Voice, and
    Google Workspace.)
    """

    _client: CloudChannelReportsServiceClient

    DEFAULT_ENDPOINT = CloudChannelReportsServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = CloudChannelReportsServiceClient.DEFAULT_MTLS_ENDPOINT

    report_path = staticmethod(CloudChannelReportsServiceClient.report_path)
    parse_report_path = staticmethod(CloudChannelReportsServiceClient.parse_report_path)
    report_job_path = staticmethod(CloudChannelReportsServiceClient.report_job_path)
    parse_report_job_path = staticmethod(
        CloudChannelReportsServiceClient.parse_report_job_path
    )
    common_billing_account_path = staticmethod(
        CloudChannelReportsServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        CloudChannelReportsServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        CloudChannelReportsServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        CloudChannelReportsServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        CloudChannelReportsServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        CloudChannelReportsServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        CloudChannelReportsServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        CloudChannelReportsServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        CloudChannelReportsServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        CloudChannelReportsServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CloudChannelReportsServiceAsyncClient: The constructed client.
        """
        return CloudChannelReportsServiceClient.from_service_account_info.__func__(CloudChannelReportsServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CloudChannelReportsServiceAsyncClient: The constructed client.
        """
        return CloudChannelReportsServiceClient.from_service_account_file.__func__(CloudChannelReportsServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return CloudChannelReportsServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> CloudChannelReportsServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            CloudChannelReportsServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(CloudChannelReportsServiceClient).get_transport_class,
        type(CloudChannelReportsServiceClient),
    )

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[str, CloudChannelReportsServiceTransport] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the cloud channel reports service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.CloudChannelReportsServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = CloudChannelReportsServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def run_report_job(
        self,
        request: Optional[Union[reports_service.RunReportJobRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Begins generation of data for a given report. The report
        identifier is a UID (for example, ``613bf59q``).

        Possible error codes:

        -  PERMISSION_DENIED: The user doesn't have access to this
           report.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: The report identifier was not found.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        contains an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        To get the results of report generation, call
        [CloudChannelReportsService.FetchReportResults][google.cloud.channel.v1.CloudChannelReportsService.FetchReportResults]
        with the
        [RunReportJobResponse.report_job][google.cloud.channel.v1.RunReportJobResponse.report_job].

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import channel_v1

            async def sample_run_report_job():
                # Create a client
                client = channel_v1.CloudChannelReportsServiceAsyncClient()

                # Initialize request argument(s)
                request = channel_v1.RunReportJobRequest(
                    name="name_value",
                )

                # Make the request
                operation = client.run_report_job(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.channel_v1.types.RunReportJobRequest, dict]]):
                The request object. Request message for
                [CloudChannelReportsService.RunReportJob][google.cloud.channel.v1.CloudChannelReportsService.RunReportJob].
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.channel_v1.types.RunReportJobResponse` Response message for
                   [CloudChannelReportsService.RunReportJob][google.cloud.channel.v1.CloudChannelReportsService.RunReportJob].

        """
        # Create or coerce a protobuf request object.
        request = reports_service.RunReportJobRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.run_report_job,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            reports_service.RunReportJobResponse,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def fetch_report_results(
        self,
        request: Optional[
            Union[reports_service.FetchReportResultsRequest, dict]
        ] = None,
        *,
        report_job: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.FetchReportResultsAsyncPager:
        r"""Retrieves data generated by
        [CloudChannelReportsService.RunReportJob][google.cloud.channel.v1.CloudChannelReportsService.RunReportJob].

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import channel_v1

            async def sample_fetch_report_results():
                # Create a client
                client = channel_v1.CloudChannelReportsServiceAsyncClient()

                # Initialize request argument(s)
                request = channel_v1.FetchReportResultsRequest(
                    report_job="report_job_value",
                )

                # Make the request
                page_result = client.fetch_report_results(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.channel_v1.types.FetchReportResultsRequest, dict]]):
                The request object. Request message for
                [CloudChannelReportsService.FetchReportResults][google.cloud.channel.v1.CloudChannelReportsService.FetchReportResults].
            report_job (:class:`str`):
                Required. The report job created by
                [CloudChannelReportsService.RunReportJob][google.cloud.channel.v1.CloudChannelReportsService.RunReportJob].
                Report_job uses the format:
                accounts/{account_id}/reportJobs/{report_job_id}

                This corresponds to the ``report_job`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_reports_service.pagers.FetchReportResultsAsyncPager:
                Response message for
                   [CloudChannelReportsService.FetchReportResults][google.cloud.channel.v1.CloudChannelReportsService.FetchReportResults].
                   Contains a tabular representation of the report
                   results.

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([report_job])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = reports_service.FetchReportResultsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if report_job is not None:
            request.report_job = report_job

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.fetch_report_results,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("report_job", request.report_job),)
            ),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.FetchReportResultsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_reports(
        self,
        request: Optional[Union[reports_service.ListReportsRequest, dict]] = None,
        *,
        parent: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListReportsAsyncPager:
        r"""Lists the reports that RunReportJob can run. These
        reports include an ID, a description, and the list of
        columns that will be in the result.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import channel_v1

            async def sample_list_reports():
                # Create a client
                client = channel_v1.CloudChannelReportsServiceAsyncClient()

                # Initialize request argument(s)
                request = channel_v1.ListReportsRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_reports(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.channel_v1.types.ListReportsRequest, dict]]):
                The request object. Request message for
                [CloudChannelReportsService.ListReports][google.cloud.channel.v1.CloudChannelReportsService.ListReports].
            parent (:class:`str`):
                Required. The resource name of the partner account to
                list available reports for. Parent uses the format:
                accounts/{account_id}

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_reports_service.pagers.ListReportsAsyncPager:
                Response message for
                   [CloudChannelReportsService.ListReports][google.cloud.channel.v1.CloudChannelReportsService.ListReports].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = reports_service.ListReportsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_reports,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListReportsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_operations(
        self,
        request: Optional[operations_pb2.ListOperationsRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operations_pb2.ListOperationsResponse:
        r"""Lists operations that match the specified filter in the request.

        Args:
            request (:class:`~.operations_pb2.ListOperationsRequest`):
                The request object. Request message for
                `ListOperations` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            ~.operations_pb2.ListOperationsResponse:
                Response message for ``ListOperations`` method.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.ListOperationsRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_operations,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_operation(
        self,
        request: Optional[operations_pb2.GetOperationRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operations_pb2.Operation:
        r"""Gets the latest state of a long-running operation.

        Args:
            request (:class:`~.operations_pb2.GetOperationRequest`):
                The request object. Request message for
                `GetOperation` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            ~.operations_pb2.Operation:
                An ``Operation`` object.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.GetOperationRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def delete_operation(
        self,
        request: Optional[operations_pb2.DeleteOperationRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes a long-running operation.

        This method indicates that the client is no longer interested
        in the operation result. It does not cancel the operation.
        If the server doesn't support this method, it returns
        `google.rpc.Code.UNIMPLEMENTED`.

        Args:
            request (:class:`~.operations_pb2.DeleteOperationRequest`):
                The request object. Request message for
                `DeleteOperation` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            None
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.DeleteOperationRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def cancel_operation(
        self,
        request: Optional[operations_pb2.CancelOperationRequest] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Starts asynchronous cancellation on a long-running operation.

        The server makes a best effort to cancel the operation, but success
        is not guaranteed.  If the server doesn't support this method, it returns
        `google.rpc.Code.UNIMPLEMENTED`.

        Args:
            request (:class:`~.operations_pb2.CancelOperationRequest`):
                The request object. Request message for
                `CancelOperation` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        Returns:
            None
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if isinstance(request, dict):
            request = operations_pb2.CancelOperationRequest(**request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.cancel_operation,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def __aenter__(self) -> "CloudChannelReportsServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("CloudChannelReportsServiceAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
from google.type import money_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.channel_v1.types import common, products

__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "PromotionalOrderType",
        "PaymentPlan",
        "PaymentType",
        "ResourceType",
        "PeriodType",
        "Offer",
        "ParameterDefinition",
        "Constraints",
        "CustomerConstraints",
        "Plan",
        "PriceByResource",
        "Price",
        "PricePhase",
        "PriceTier",
        "Period",
    },
)


class PromotionalOrderType(proto.Enum):
    r"""Constraints type for Promotional offers.

    Values:
        PROMOTIONAL_TYPE_UNSPECIFIED (0):
            Not used.
        NEW_UPGRADE (1):
            Order used for new customers, trial
            conversions and upgrades.
        TRANSFER (2):
            All orders for transferring an existing
            customer.
        PROMOTION_SWITCH (3):
            Orders for modifying an existing customer's
            promotion on the same SKU.
    """
    PROMOTIONAL_TYPE_UNSPECIFIED = 0
    NEW_UPGRADE = 1
    TRANSFER = 2
    PROMOTION_SWITCH = 3


class PaymentPlan(proto.Enum):
    r"""Describes how the reseller will be billed.

    Values:
        PAYMENT_PLAN_UNSPECIFIED (0):
            Not used.
        COMMITMENT (1):
            Commitment.
        FLEXIBLE (2):
            No commitment.
        FREE (3):
            Free.
        TRIAL (4):
            Trial.
        OFFLINE (5):
            Price and ordering not available through API.
    """
    PAYMENT_PLAN_UNSPECIFIED = 0
    COMMITMENT = 1
    FLEXIBLE = 2
    FREE = 3
    TRIAL = 4
    OFFLINE = 5


class PaymentType(proto.Enum):
    r"""Specifies when the payment needs to happen.

    Values:
        PAYMENT_TYPE_UNSPECIFIED (0):
            Not used.
        PREPAY (1):
            Prepay. Amount has to be paid before service
            is rendered.
        POSTPAY (2):
            Postpay. Reseller is charged at the end of
            the Payment cycle.
    """
    PAYMENT_TYPE_UNSPECIFIED = 0
    PREPAY = 1
    POSTPAY = 2


class ResourceType(proto.Enum):
    r"""Represents the type for a monetizable resource(any entity on
    which billing happens). For example, this could be MINUTES for
    Google Voice and GB for Google Drive. One SKU can map to
    multiple monetizable resources.

    Values:
        RESOURCE_TYPE_UNSPECIFIED (0):
            Not used.
        SEAT (1):
            Seat.
        MAU (2):
            Monthly active user.
        GB (3):
            GB (used for storage SKUs).
        LICENSED_USER (4):
            Active licensed users(for Voice SKUs).
        MINUTES (5):
            Voice usage.
        IAAS_USAGE (6):
            For IaaS SKUs like Google Cloud, monetization
            is based on usage accrued on your billing
            account irrespective of the type of monetizable
            resource. This enum represents an aggregated
            resource/container for all usage SKUs on a
            billing account. Currently, only applicable to
            Google Cloud.
        SUBSCRIPTION (7):
            For Google Cloud subscriptions like Anthos or
            SAP.
    """
    RESOURCE_TYPE_UNSPECIFIED = 0
    SEAT = 1
    MAU = 2
    GB = 3
    LICENSED_USER = 4
    MINUTES = 5
    IAAS_USAGE = 6
    SUBSCRIPTION = 7


class PeriodType(proto.Enum):
    r"""Period Type.

    Values:
        PERIOD_TYPE_UNSPECIFIED (0):
            Not used.
        DAY (1):
            Day.
        MONTH (2):
            Month.
        YEAR (3):
            Year.
    """
    PERIOD_TYPE_UNSPECIFIED = 0
    DAY = 1
    MONTH = 2
    YEAR = 3


class Offer(proto.Message):
    r"""Represents an offer made to resellers for purchase. An offer is
    associated with a [Sku][google.cloud.channel.v1.Sku], has a plan for
    payment, a price, and defines the constraints for buying.

    Attributes:
        name (str):
            Resource Name of the Offer. Format:
            accounts/{account_id}/offers/{offer_id}
        marketing_info (google.cloud.channel_v1.types.MarketingInfo):
            Marketing information for the Offer.
        sku (google.cloud.channel_v1.types.Sku):
            SKU the offer is associated with.
        plan (google.cloud.channel_v1.types.Plan):
            Describes the payment plan for the Offer.
        constraints (google.cloud.channel_v1.types.Constraints):
            Constraints on transacting the Offer.
        price_by_resources (MutableSequence[google.cloud.channel_v1.types.PriceByResource]):
            Price for each monetizable resource type.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Start of the Offer validity time.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. End of the Offer validity time.
        parameter_definitions (MutableSequence[google.cloud.channel_v1.types.ParameterDefinition]):
            Parameters required to use current Offer to
            purchase.
        deal_code (str):
            The deal code of the offer to get a special
            promotion or discount.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    marketing_info: products.MarketingInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=products.MarketingInfo,
    )
    sku: products.Sku = proto.Field(
        proto.MESSAGE,
        number=3,
        message=products.Sku,
    )
    plan: "Plan" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="Plan",
    )
    constraints: "Constraints" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="Constraints",
    )
    price_by_resources: MutableSequence["PriceByResource"] = proto.RepeatedField(
        proto.MESSAGE,
        number=6,
        message="PriceByResource",
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=8,
        message=timestamp_pb2.Timestamp,
    )
    parameter_definitions: MutableSequence["ParameterDefinition"] = proto.RepeatedField(
        proto.MESSAGE,
        number=9,
        message="ParameterDefinition",
    )
    deal_code: str = proto.Field(
        proto.STRING,
        number=12,
    )


class ParameterDefinition(proto.Message):
    r"""Parameter's definition. Specifies what parameter is required
    to use the current Offer to purchase.

    Attributes:
        name (str):
            Name of the parameter.
        parameter_type (google.cloud.channel_v1.types.ParameterDefinition.ParameterType):
            Data type of the parameter. Minimal value,
            Maximum value and allowed values will use
            specified data type here.
        min_value (google.cloud.channel_v1.types.Value):
            Minimal value of the parameter, if
            applicable. Inclusive. For example, minimal
            commitment when purchasing Anthos is 0.01.
            Applicable to INT64 and DOUBLE parameter types.
        max_value (google.cloud.channel_v1.types.Value):
            Maximum value of the parameter, if
            applicable. Inclusive. For example, maximum
            seats when purchasing Google Workspace Business
            Standard. Applicable to INT64 and DOUBLE
            parameter types.
        allowed_values (MutableSequence[google.cloud.channel_v1.types.Value]):
            If not empty, parameter values must be drawn from this list.
            For example, [us-west1, us-west2, ...] Applicable to STRING
            parameter type.
        optional (bool):
            If set to true, parameter is optional to
            purchase this Offer.
    """

    class ParameterType(proto.Enum):
        r"""Data type of the parameter.

        Values:
            PARAMETER_TYPE_UNSPECIFIED (0):
                Not used.
            INT64 (1):
                Int64 type.
            STRING (2):
                String type.
            DOUBLE (3):
                Double type.
            BOOLEAN (4):
                Boolean type.
        """
        PARAMETER_TYPE_UNSPECIFIED = 0
        INT64 = 1
        STRING = 2
        DOUBLE = 3
        BOOLEAN = 4

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    parameter_type: ParameterType = proto.Field(
        proto.ENUM,
        number=2,
        enum=ParameterType,
    )
    min_value: common.Value = proto.Field(
        proto.MESSAGE,
        number=3,
        message=common.Value,
    )
    max_value: common.Value = proto.Field(
        proto.MESSAGE,
        number=4,
        message=common.Value,
    )
    allowed_values: MutableSequence[common.Value] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=common.Value,
    )
    optional: bool = proto.Field(
        proto.BOOL,
        number=6,
    )


class Constraints(proto.Message):
    r"""Represents the constraints for buying the Offer.

    Attributes:
        customer_constraints (google.cloud.channel_v1.types.CustomerConstraints):
            Represents constraints required to purchase
            the Offer for a customer.
    """

    customer_constraints: "CustomerConstraints" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="CustomerConstraints",
    )


class CustomerConstraints(proto.Message):
    r"""Represents constraints required to purchase the Offer for a
    customer.

    Attributes:
        allowed_regions (MutableSequence[str]):
            Allowed geographical regions of the customer.
        allowed_customer_types (MutableSequence[google.cloud.channel_v1.types.CloudIdentityInfo.CustomerType]):
            Allowed Customer Type.
        promotional_order_types (MutableSequence[google.cloud.channel_v1.types.PromotionalOrderType]):
            Allowed Promotional Order Type. Present for
            Promotional offers.
    """

    allowed_regions: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=1,
    )
    allowed_customer_types: MutableSequence[
        common.CloudIdentityInfo.CustomerType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=2,
        enum=common.CloudIdentityInfo.CustomerType,
    )
    promotional_order_types: MutableSequence[
        "PromotionalOrderType"
    ] = proto.RepeatedField(
        proto.ENUM,
        number=3,
        enum="PromotionalOrderType",
    )


class Plan(proto.Message):
    r"""The payment plan for the Offer. Describes how to make a
    payment.

    Attributes:
        payment_plan (google.cloud.channel_v1.types.PaymentPlan):
            Describes how a reseller will be billed.
        payment_type (google.cloud.channel_v1.types.PaymentType):
            Specifies when the payment needs to happen.
        payment_cycle (google.cloud.channel_v1.types.Period):
            Describes how frequently the reseller will be
            billed, such as once per month.
        trial_period (google.cloud.channel_v1.types.Period):
            Present for Offers with a trial period.
            For trial-only Offers, a paid service needs to
            start before the trial period ends for continued
            service.
            For Regular Offers with a trial period, the
            regular pricing goes into effect when trial
            period ends, or if paid service is started
            before the end of the trial period.
        billing_account (str):
            Reseller Billing account to charge after an
            offer transaction. Only present for Google Cloud
            offers.
    """

    payment_plan: "PaymentPlan" = proto.Field(
        proto.ENUM,
        number=1,
        enum="PaymentPlan",
    )
    payment_type: "PaymentType" = proto.Field(
        proto.ENUM,
        number=2,
        enum="PaymentType",
    )
    payment_cycle: "Period" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="Period",
    )
    trial_period: "Period" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="Period",
    )
    billing_account: str = proto.Field(
        proto.STRING,
        number=5,
    )


class PriceByResource(proto.Message):
    r"""Represents price by resource type.

    Attributes:
        resource_type (google.cloud.channel_v1.types.ResourceType):
            Resource Type. Example: SEAT
        price (google.cloud.channel_v1.types.Price):
            Price of the Offer. Present if there are no
            price phases.
        price_phases (MutableSequence[google.cloud.channel_v1.types.PricePhase]):
            Specifies the price by time range.
    """

    resource_type: "ResourceType" = proto.Field(
        proto.ENUM,
        number=1,
        enum="ResourceType",
    )
    price: "Price" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="Price",
    )
    price_phases: MutableSequence["PricePhase"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="PricePhase",
    )


class Price(proto.Message):
    r"""Represents the price of the Offer.

    Attributes:
        base_price (google.type.money_pb2.Money):
            Base price.
        discount (float):
            Discount percentage, represented as decimal.
            For example, a 20% discount will be represent as
            0.2.
        effective_price (google.type.money_pb2.Money):
            Effective Price after applying the discounts.
        external_price_uri (str):
            Link to external price list, such as link to
            Google Voice rate card.
    """

    base_price: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=1,
        message=money_pb2.Money,
    )
    discount: float = proto.Field(
        proto.DOUBLE,
        number=2,
    )
    effective_price: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=3,
        message=money_pb2.Money,
    )
    external_price_uri: str = proto.Field(
        proto.STRING,
        number=4,
    )


class PricePhase(proto.Message):
    r"""Specifies the price by the duration of months.
    For example, a 20% discount for the first six months, then a 10%
    discount starting on the seventh month.

    Attributes:
        period_type (google.cloud.channel_v1.types.PeriodType):
            Defines the phase period type.
        first_period (int):
            Defines first period for the phase.
        last_period (int):
            Defines first period for the phase.
        price (google.cloud.channel_v1.types.Price):
            Price of the phase. Present if there are no
            price tiers.
        price_tiers (MutableSequence[google.cloud.channel_v1.types.PriceTier]):
            Price by the resource tiers.
    """

    period_type: "PeriodType" = proto.Field(
        proto.ENUM,
        number=1,
        enum="PeriodType",
    )
    first_period: int = proto.Field(
        proto.INT32,
        number=2,
    )
    last_period: int = proto.Field(
        proto.INT32,
        number=3,
    )
    price: "Price" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="Price",
    )
    price_tiers: MutableSequence["PriceTier"] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message="PriceTier",
    )


class PriceTier(proto.Message):
    r"""Defines price at resource tier level. For example, an offer with
    following definition :

    -  Tier 1: Provide 25% discount for all seats between 1 and 25.
    -  Tier 2: Provide 10% discount for all seats between 26 and 100.
    -  Tier 3: Provide flat 15% discount for all seats above 100.

    Each of these tiers is represented as a PriceTier.

    Attributes:
        first_resource (int):
            First resource for which the tier price
            applies.
        last_resource (int):
            Last resource for which the tier price
            applies.
        price (google.cloud.channel_v1.types.Price):
            Price of the tier.
    """

    first_resource: int = proto.Field(
        proto.INT32,
        number=1,
    )
    last_resource: int = proto.Field(
        proto.INT32,
        number=2,
    )
    price: "Price" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="Price",
    )


class Period(proto.Message):
    r"""Represents period in days/months/years.

    Attributes:
        duration (int):
            Total duration of Period Type defined.
        period_type (google.cloud.channel_v1.types.PeriodType):
            Period Type.
    """

    duration: int = proto.Field(
        proto.INT32,
        number=1,
    )
    period_type: "PeriodType" = proto.Field(
        proto.ENUM,
        number=2,
        enum="PeriodType",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import any_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "EduData",
        "CloudIdentityInfo",
        "Value",
        "AdminUser",
    },
)


class EduData(proto.Message):
    r"""Required Edu Attributes

    Attributes:
        institute_type (google.cloud.channel_v1.types.EduData.InstituteType):
            Designated institute type of customer.
        institute_size (google.cloud.channel_v1.types.EduData.InstituteSize):
            Size of the institute.
        website (str):
            Web address for the edu customer's
            institution.
    """

    class InstituteType(proto.Enum):
        r"""Enum to specify the institute type.

        Values:
            INSTITUTE_TYPE_UNSPECIFIED (0):
                Not used.
            K12 (1):
                Elementary/Secondary Schools & Districts
            UNIVERSITY (2):
                Higher Education Universities & Colleges
        """
        INSTITUTE_TYPE_UNSPECIFIED = 0
        K12 = 1
        UNIVERSITY = 2

    class InstituteSize(proto.Enum):
        r"""Number of students and staff the institute has.

        Values:
            INSTITUTE_SIZE_UNSPECIFIED (0):
                Not used.
            SIZE_1_100 (1):
                1 - 100
            SIZE_101_500 (2):
                101 - 500
            SIZE_501_1000 (3):
                501 - 1,000
            SIZE_1001_2000 (4):
                1,001 - 2,000
            SIZE_2001_5000 (5):
                2,001 - 5,000
            SIZE_5001_10000 (6):
                5,001 - 10,000
            SIZE_10001_OR_MORE (7):
                10,001 +
        """
        INSTITUTE_SIZE_UNSPECIFIED = 0
        SIZE_1_100 = 1
        SIZE_101_500 = 2
        SIZE_501_1000 = 3
        SIZE_1001_2000 = 4
        SIZE_2001_5000 = 5
        SIZE_5001_10000 = 6
        SIZE_10001_OR_MORE = 7

    institute_type: InstituteType = proto.Field(
        proto.ENUM,
        number=1,
        enum=InstituteType,
    )
    institute_size: InstituteSize = proto.Field(
        proto.ENUM,
        number=2,
        enum=InstituteSize,
    )
    website: str = proto.Field(
        proto.STRING,
        number=3,
    )


class CloudIdentityInfo(proto.Message):
    r"""Cloud Identity information for the Cloud Channel Customer.

    Attributes:
        customer_type (google.cloud.channel_v1.types.CloudIdentityInfo.CustomerType):
            CustomerType indicates verification type
            needed for using services.
        primary_domain (str):
            Output only. The primary domain name.
        is_domain_verified (bool):
            Output only. Whether the domain is verified. This field is
            not returned for a Customer's cloud_identity_info resource.
            Partners can use the domains.get() method of the Workspace
            SDK's Directory API, or listen to the
            PRIMARY_DOMAIN_VERIFIED Pub/Sub event in to track domain
            verification of their resolve Workspace customers.
        alternate_email (str):
            The alternate email.
        phone_number (str):
            Phone number associated with the Cloud
            Identity.
        language_code (str):
            Language code.
        admin_console_uri (str):
            Output only. URI of Customer's Admin console
            dashboard.
        edu_data (google.cloud.channel_v1.types.EduData):
            Edu information about the customer.
    """

    class CustomerType(proto.Enum):
        r"""CustomerType of the customer

        Values:
            CUSTOMER_TYPE_UNSPECIFIED (0):
                Not used.
            DOMAIN (1):
                Domain-owning customer which needs domain
                verification to use services.
            TEAM (2):
                Team customer which needs email verification
                to use services.
        """
        CUSTOMER_TYPE_UNSPECIFIED = 0
        DOMAIN = 1
        TEAM = 2

    customer_type: CustomerType = proto.Field(
        proto.ENUM,
        number=1,
        enum=CustomerType,
    )
    primary_domain: str = proto.Field(
        proto.STRING,
        number=9,
    )
    is_domain_verified: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    alternate_email: str = proto.Field(
        proto.STRING,
        number=6,
    )
    phone_number: str = proto.Field(
        proto.STRING,
        number=7,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=8,
    )
    admin_console_uri: str = proto.Field(
        proto.STRING,
        number=10,
    )
    edu_data: "EduData" = proto.Field(
        proto.MESSAGE,
        number=22,
        message="EduData",
    )


class Value(proto.Message):
    r"""Data type and value of a parameter.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        int64_value (int):
            Represents an int64 value.

            This field is a member of `oneof`_ ``kind``.
        string_value (str):
            Represents a string value.

            This field is a member of `oneof`_ ``kind``.
        double_value (float):
            Represents a double value.

            This field is a member of `oneof`_ ``kind``.
        proto_value (google.protobuf.any_pb2.Any):
            Represents an 'Any' proto value.

            This field is a member of `oneof`_ ``kind``.
        bool_value (bool):
            Represents a boolean value.

            This field is a member of `oneof`_ ``kind``.
    """

    int64_value: int = proto.Field(
        proto.INT64,
        number=1,
        oneof="kind",
    )
    string_value: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="kind",
    )
    double_value: float = proto.Field(
        proto.DOUBLE,
        number=3,
        oneof="kind",
    )
    proto_value: any_pb2.Any = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="kind",
        message=any_pb2.Any,
    )
    bool_value: bool = proto.Field(
        proto.BOOL,
        number=5,
        oneof="kind",
    )


class AdminUser(proto.Message):
    r"""Information needed to create an Admin User for Google
    Workspace.

    Attributes:
        email (str):
            Primary email of the admin user.
        given_name (str):
            Given name of the admin user.
        family_name (str):
            Family name of the admin user.
    """

    email: str = proto.Field(
        proto.STRING,
        number=1,
    )
    given_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    family_name: str = proto.Field(
        proto.STRING,
        number=3,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
from google.type import postal_address_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.channel_v1.types import common

__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "Customer",
        "ContactInfo",
    },
)


class Customer(proto.Message):
    r"""Entity representing a customer of a reseller or distributor.

    Attributes:
        name (str):
            Output only. Resource name of the customer. Format:
            accounts/{account_id}/customers/{customer_id}
        org_display_name (str):
            Required. Name of the organization that the
            customer entity represents.
        org_postal_address (google.type.postal_address_pb2.PostalAddress):
            Required. The organization address for the
            customer. To enforce US laws and embargoes, we
            require a region, postal code, and address
            lines. You must provide valid addresses for
            every customer. To set the customer's language,
            use the Customer-level language code.
        primary_contact_info (google.cloud.channel_v1.types.ContactInfo):
            Primary contact info.
        alternate_email (str):
            Secondary contact email. You need to provide
            an alternate email to create different domains
            if a primary contact email already exists. Users
            will receive a notification with credentials
            when you create an admin.google.com account.
            Secondary emails are also recovery email
            addresses. Alternate emails are optional when
            you create Team customers.
        domain (str):
            Required. The customer's primary domain. Must
            match the primary contact email's domain.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when the customer was
            created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when the customer was
            updated.
        cloud_identity_id (str):
            Output only. The customer's Cloud Identity ID
            if the customer has a Cloud Identity resource.
        language_code (str):
            Optional. The BCP-47 language code, such as "en-US" or
            "sr-Latn". For more information, see
            https://www.unicode.org/reports/tr35/#Unicode_locale_identifier.
        cloud_identity_info (google.cloud.channel_v1.types.CloudIdentityInfo):
            Output only. Cloud Identity information for
            the customer. Populated only if a Cloud Identity
            account exists for this customer.
        channel_partner_id (str):
            Cloud Identity ID of the customer's channel
            partner. Populated only if a channel partner
            exists for this customer.
        correlation_id (str):
            Optional. External CRM ID for the customer.
            Populated only if a CRM ID exists for this
            customer.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    org_display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    org_postal_address: postal_address_pb2.PostalAddress = proto.Field(
        proto.MESSAGE,
        number=3,
        message=postal_address_pb2.PostalAddress,
    )
    primary_contact_info: "ContactInfo" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="ContactInfo",
    )
    alternate_email: str = proto.Field(
        proto.STRING,
        number=5,
    )
    domain: str = proto.Field(
        proto.STRING,
        number=6,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=8,
        message=timestamp_pb2.Timestamp,
    )
    cloud_identity_id: str = proto.Field(
        proto.STRING,
        number=9,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=10,
    )
    cloud_identity_info: common.CloudIdentityInfo = proto.Field(
        proto.MESSAGE,
        number=12,
        message=common.CloudIdentityInfo,
    )
    channel_partner_id: str = proto.Field(
        proto.STRING,
        number=13,
    )
    correlation_id: str = proto.Field(
        proto.STRING,
        number=14,
    )


class ContactInfo(proto.Message):
    r"""Contact information for a customer account.

    Attributes:
        first_name (str):
            The customer account contact's first name.
            Optional for Team customers.
        last_name (str):
            The customer account contact's last name.
            Optional for Team customers.
        display_name (str):
            Output only. The customer account contact's
            display name, formatted as a combination of the
            customer's first and last name.
        email (str):
            The customer account's contact email.
            Required for entitlements that create
            admin.google.com accounts, and serves as the
            customer's username for those accounts. Use this
            email to invite Team customers.
        title (str):
            Optional. The customer account contact's job
            title.
        phone (str):
            The customer account's contact phone number.
    """

    first_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    last_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=4,
    )
    email: str = proto.Field(
        proto.STRING,
        number=5,
    )
    title: str = proto.Field(
        proto.STRING,
        number=6,
    )
    phone: str = proto.Field(
        proto.STRING,
        number=7,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "OperationMetadata",
    },
)


class OperationMetadata(proto.Message):
    r"""Provides contextual information about a
    [google.longrunning.Operation][google.longrunning.Operation].

    Attributes:
        operation_type (google.cloud.channel_v1.types.OperationMetadata.OperationType):
            The RPC that initiated this Long Running
            Operation.
    """

    class OperationType(proto.Enum):
        r"""RPCs that return a Long Running Operation.

        Values:
            OPERATION_TYPE_UNSPECIFIED (0):
                Not used.
            CREATE_ENTITLEMENT (1):
                Long Running Operation was triggered by
                CreateEntitlement.
            CHANGE_RENEWAL_SETTINGS (3):
                Long Running Operation was triggered by
                ChangeRenewalSettings.
            START_PAID_SERVICE (5):
                Long Running Operation was triggered by
                StartPaidService.
            ACTIVATE_ENTITLEMENT (7):
                Long Running Operation was triggered by
                ActivateEntitlement.
            SUSPEND_ENTITLEMENT (8):
                Long Running Operation was triggered by
                SuspendEntitlement.
            CANCEL_ENTITLEMENT (9):
                Long Running Operation was triggered by
                CancelEntitlement.
            TRANSFER_ENTITLEMENTS (10):
                Long Running Operation was triggered by
                TransferEntitlements.
            TRANSFER_ENTITLEMENTS_TO_GOOGLE (11):
                Long Running Operation was triggered by
                TransferEntitlementsToGoogle.
            CHANGE_OFFER (14):
                Long Running Operation was triggered by
                ChangeOffer.
            CHANGE_PARAMETERS (15):
                Long Running Operation was triggered by
                ChangeParameters.
            PROVISION_CLOUD_IDENTITY (16):
                Long Running Operation was triggered by
                ProvisionCloudIdentity.
        """
        OPERATION_TYPE_UNSPECIFIED = 0
        CREATE_ENTITLEMENT = 1
        CHANGE_RENEWAL_SETTINGS = 3
        START_PAID_SERVICE = 5
        ACTIVATE_ENTITLEMENT = 7
        SUSPEND_ENTITLEMENT = 8
        CANCEL_ENTITLEMENT = 9
        TRANSFER_ENTITLEMENTS = 10
        TRANSFER_ENTITLEMENTS_TO_GOOGLE = 11
        CHANGE_OFFER = 14
        CHANGE_PARAMETERS = 15
        PROVISION_CLOUD_IDENTITY = 16

    operation_type: OperationType = proto.Field(
        proto.ENUM,
        number=1,
        enum=OperationType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.channel_v1.types import common, offers, products

__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "Entitlement",
        "Parameter",
        "AssociationInfo",
        "ProvisionedService",
        "CommitmentSettings",
        "RenewalSettings",
        "TrialSettings",
        "TransferableSku",
        "TransferEligibility",
    },
)


class Entitlement(proto.Message):
    r"""An entitlement is a representation of a customer's ability to
    use a service.

    Attributes:
        name (str):
            Output only. Resource name of an entitlement in the form:
            accounts/{account_id}/customers/{customer_id}/entitlements/{entitlement_id}.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time at which the
            entitlement is created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time at which the
            entitlement is updated.
        offer (str):
            Required. The offer resource name for which the entitlement
            is to be created. Takes the form:
            accounts/{account_id}/offers/{offer_id}.
        commitment_settings (google.cloud.channel_v1.types.CommitmentSettings):
            Commitment settings for a commitment-based
            Offer. Required for commitment based offers.
        provisioning_state (google.cloud.channel_v1.types.Entitlement.ProvisioningState):
            Output only. Current provisioning state of
            the entitlement.
        provisioned_service (google.cloud.channel_v1.types.ProvisionedService):
            Output only. Service provisioning details for
            the entitlement.
        suspension_reasons (MutableSequence[google.cloud.channel_v1.types.Entitlement.SuspensionReason]):
            Output only. Enumerable of all current
            suspension reasons for an entitlement.
        purchase_order_id (str):
            Optional. This purchase order (PO)
            information is for resellers to use for their
            company tracking usage. If a purchaseOrderId
            value is given, it appears in the API responses
            and shows up in the invoice. The property
            accepts up to 80 plain text characters. This is
            only supported for Google Workspace
            entitlements.
        trial_settings (google.cloud.channel_v1.types.TrialSettings):
            Output only. Settings for trial offers.
        association_info (google.cloud.channel_v1.types.AssociationInfo):
            Association information to other
            entitlements.
        parameters (MutableSequence[google.cloud.channel_v1.types.Parameter]):
            Extended entitlement parameters. When creating an
            entitlement, valid parameter names and values are defined in
            the
            [Offer.parameter_definitions][google.cloud.channel.v1.Offer.parameter_definitions].

            For Google Workspace, the following Parameters may be
            accepted as input:

            -  max_units: The maximum assignable units for a flexible
               offer

            OR

            -  num_units: The total commitment for commitment-based
               offers

            The response may additionally include the following
            output-only Parameters:

            -  assigned_units: The number of licenses assigned to users.

            For Google Cloud billing subaccounts, the following
            Parameter may be accepted as input:

            -  display_name: The display name of the billing subaccount.
        billing_account (str):
            Optional. The billing account resource name
            that is used to pay for this entitlement.
    """

    class ProvisioningState(proto.Enum):
        r"""Indicates the current provisioning state of the entitlement.

        Values:
            PROVISIONING_STATE_UNSPECIFIED (0):
                Not used.
            ACTIVE (1):
                The entitlement is currently active.
            SUSPENDED (5):
                The entitlement is currently suspended.
        """
        PROVISIONING_STATE_UNSPECIFIED = 0
        ACTIVE = 1
        SUSPENDED = 5

    class SuspensionReason(proto.Enum):
        r"""Suspension reason for an entitlement if
        [provisioning_state][google.cloud.channel.v1.Entitlement.provisioning_state]
        = SUSPENDED.

        Values:
            SUSPENSION_REASON_UNSPECIFIED (0):
                Not used.
            RESELLER_INITIATED (1):
                Entitlement was manually suspended by the
                Reseller.
            TRIAL_ENDED (2):
                Trial ended.
            RENEWAL_WITH_TYPE_CANCEL (3):
                Entitlement renewal was canceled.
            PENDING_TOS_ACCEPTANCE (4):
                Entitlement was automatically suspended on
                creation for pending ToS acceptance on customer.
            OTHER (100):
                Other reasons (internal reasons, abuse,
                etc.).
        """
        SUSPENSION_REASON_UNSPECIFIED = 0
        RESELLER_INITIATED = 1
        TRIAL_ENDED = 2
        RENEWAL_WITH_TYPE_CANCEL = 3
        PENDING_TOS_ACCEPTANCE = 4
        OTHER = 100

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )
    offer: str = proto.Field(
        proto.STRING,
        number=8,
    )
    commitment_settings: "CommitmentSettings" = proto.Field(
        proto.MESSAGE,
        number=12,
        message="CommitmentSettings",
    )
    provisioning_state: ProvisioningState = proto.Field(
        proto.ENUM,
        number=13,
        enum=ProvisioningState,
    )
    provisioned_service: "ProvisionedService" = proto.Field(
        proto.MESSAGE,
        number=16,
        message="ProvisionedService",
    )
    suspension_reasons: MutableSequence[SuspensionReason] = proto.RepeatedField(
        proto.ENUM,
        number=18,
        enum=SuspensionReason,
    )
    purchase_order_id: str = proto.Field(
        proto.STRING,
        number=19,
    )
    trial_settings: "TrialSettings" = proto.Field(
        proto.MESSAGE,
        number=21,
        message="TrialSettings",
    )
    association_info: "AssociationInfo" = proto.Field(
        proto.MESSAGE,
        number=23,
        message="AssociationInfo",
    )
    parameters: MutableSequence["Parameter"] = proto.RepeatedField(
        proto.MESSAGE,
        number=26,
        message="Parameter",
    )
    billing_account: str = proto.Field(
        proto.STRING,
        number=28,
    )


class Parameter(proto.Message):
    r"""Definition for extended entitlement parameters.

    Attributes:
        name (str):
            Name of the parameter.
        value (google.cloud.channel_v1.types.Value):
            Value of the parameter.
        editable (bool):
            Output only. Specifies whether this parameter is allowed to
            be changed. For example, for a Google Workspace Business
            Starter entitlement in commitment plan, num_units is
            editable when entitlement is active.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    value: common.Value = proto.Field(
        proto.MESSAGE,
        number=2,
        message=common.Value,
    )
    editable: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class AssociationInfo(proto.Message):
    r"""Association links that an entitlement has to other
    entitlements.

    Attributes:
        base_entitlement (str):
            The name of the base entitlement, for which
            this entitlement is an add-on.
    """

    base_entitlement: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ProvisionedService(proto.Message):
    r"""Service provisioned for an entitlement.

    Attributes:
        provisioning_id (str):
            Output only. Provisioning ID of the
            entitlement. For Google Workspace, this is the
            underlying Subscription ID. For Google Cloud,
            this is the Billing Account ID of the billing
            subaccount.
        product_id (str):
            Output only. The product pertaining to the
            provisioning resource as specified in the Offer.
        sku_id (str):
            Output only. The SKU pertaining to the
            provisioning resource as specified in the Offer.
    """

    provisioning_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    product_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    sku_id: str = proto.Field(
        proto.STRING,
        number=3,
    )


class CommitmentSettings(proto.Message):
    r"""Commitment settings for commitment-based offers.

    Attributes:
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Commitment start timestamp.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Commitment end timestamp.
        renewal_settings (google.cloud.channel_v1.types.RenewalSettings):
            Optional. Renewal settings applicable for a
            commitment-based Offer.
    """

    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    renewal_settings: "RenewalSettings" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="RenewalSettings",
    )


class RenewalSettings(proto.Message):
    r"""Renewal settings for renewable Offers.

    Attributes:
        enable_renewal (bool):
            If false, the plan will be completed at the
            end date.
        resize_unit_count (bool):
            If true and enable_renewal = true, the unit (for example
            seats or licenses) will be set to the number of active units
            at renewal time.
        payment_plan (google.cloud.channel_v1.types.PaymentPlan):
            Describes how a reseller will be billed.
        payment_cycle (google.cloud.channel_v1.types.Period):
            Describes how frequently the reseller will be
            billed, such as once per month.
    """

    enable_renewal: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    resize_unit_count: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    payment_plan: offers.PaymentPlan = proto.Field(
        proto.ENUM,
        number=5,
        enum=offers.PaymentPlan,
    )
    payment_cycle: offers.Period = proto.Field(
        proto.MESSAGE,
        number=6,
        message=offers.Period,
    )


class TrialSettings(proto.Message):
    r"""Settings for trial offers.

    Attributes:
        trial (bool):
            Determines if the entitlement is in a trial or not:

            -  ``true`` - The entitlement is in trial.
            -  ``false`` - The entitlement is not in trial.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Date when the trial ends. The value is in milliseconds using
            the UNIX Epoch format. See an example `Epoch
            converter <https://www.epochconverter.com>`__.
    """

    trial: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )


class TransferableSku(proto.Message):
    r"""TransferableSku represents information a reseller needs to
    view existing provisioned services for a customer that they do
    not own. Read-only.

    Attributes:
        transfer_eligibility (google.cloud.channel_v1.types.TransferEligibility):
            Describes the transfer eligibility of a SKU.
        sku (google.cloud.channel_v1.types.Sku):
            The SKU pertaining to the provisioning
            resource as specified in the Offer.
        legacy_sku (google.cloud.channel_v1.types.Sku):
            Optional. The customer to transfer has an
            entitlement with the populated legacy SKU.
    """

    transfer_eligibility: "TransferEligibility" = proto.Field(
        proto.MESSAGE,
        number=9,
        message="TransferEligibility",
    )
    sku: products.Sku = proto.Field(
        proto.MESSAGE,
        number=11,
        message=products.Sku,
    )
    legacy_sku: products.Sku = proto.Field(
        proto.MESSAGE,
        number=12,
        message=products.Sku,
    )


class TransferEligibility(proto.Message):
    r"""Specifies transfer eligibility of a SKU.

    Attributes:
        is_eligible (bool):
            Whether reseller is eligible to transfer the
            SKU.
        description (str):
            Localized description if reseller is not
            eligible to transfer the SKU.
        ineligibility_reason (google.cloud.channel_v1.types.TransferEligibility.Reason):
            Specified the reason for ineligibility.
    """

    class Reason(proto.Enum):
        r"""Reason of ineligibility.

        Values:
            REASON_UNSPECIFIED (0):
                Not used.
            PENDING_TOS_ACCEPTANCE (1):
                Reseller needs to accept TOS before
                transferring the SKU.
            SKU_NOT_ELIGIBLE (2):
                Reseller not eligible to sell the SKU.
            SKU_SUSPENDED (3):
                SKU subscription is suspended
            CHANNEL_PARTNER_NOT_AUTHORIZED_FOR_SKU (4):
                The reseller is not authorized to transact on
                this Product. See
                https://support.google.com/channelservices/answer/9759265
        """
        REASON_UNSPECIFIED = 0
        PENDING_TOS_ACCEPTANCE = 1
        SKU_NOT_ELIGIBLE = 2
        SKU_SUSPENDED = 3
        CHANNEL_PARTNER_NOT_AUTHORIZED_FOR_SKU = 4

    is_eligible: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    description: str = proto.Field(
        proto.STRING,
        number=2,
    )
    ineligibility_reason: Reason = proto.Field(
        proto.ENUM,
        number=3,
        enum=Reason,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
